/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is WebDavBrowser.
 *
 * The Initial Developer of the Original Code is
 *   Rafa Malinowski, Pozna Supercomputing And Networing Center
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
const nsIWebDavResource = Components.interfaces.nsIWebDavResource;

/**
 * Item on the tree. Represend nsIWebDavResource
 */
WebDavTreeItem.prototype = {
	_webDavTreeView: null,
	_webDavResource: null,
	_items: null,
	_parent: null,
	_level: -1,
	_index: -1,
	_opened: 0,
	_removing: false,
	_sortBy: "name",
	_sortIncreasing: true,
	
	// nsIWebDavResourceObserver
	
	onStatusChanged: function(resource) {
		if (resource.loaded == nsIWebDavResource.WEBDAV_LOADED_CREATING_FAILED) {
			// remove it becouse it doesn't exists, and it wont
			this._webDavTreeView.removeItems(this._index, this.subItemsCount + 1);
			if (this._parent)
				this._parent.subItemRemoved(this);
			return;
		}

		this._webDavTreeView.itemUpdated(this.index);
	},
	
	onLoaded: function(resource) {
	},

	onRemoved: function(resource) {
		// remove it becouse it doesn't exists now
		this._webDavTreeView.removeItems(this._index, this.subItemsCount + 1);
		
		this._items = new Array();
		if (this._parent)
			this._parent.subItemRemoved(this);
	},
	
	onSubResourceAdded: function(resource, subResource) {
		if (!this._opened)
			return;

		var wevDavTreeItem = new WebDavTreeItem(this._webDavTreeView, this, subResource);
		
		var i = 0;
		for (; i < this._items.length; i++)
			if (this._compareItems(this, this._items[i], wevDavTreeItem) == 1)
				break;

		this._items.splice(i, 0, wevDavTreeItem);
		this._webDavTreeView.addItem(this.level + 1, this.index + i + 1, wevDavTreeItem);
	},

	onSubResourcesLoaded: function(resource) {
		this._webDavTreeView.itemUpdated(this.index);
	},

	onCreated: function(resource) {
		this._webDavTreeView.itemUpdated(this.index);
	},
	
	onDebugMessage: function(message) {
		alert(message);
	},

	onError: function(resource, action, errorCode) {
		var error;
		var reason;

		switch (action) {
			case nsIWebDavResource.WEBDAV_ACTION_LOAD:
				error = "Unable to load resource " + resource.path;
				break;
			case nsIWebDavResource.WEBDAV_ACTION_CREATE:
				error = "Unable to create resource in " + resource.path;
				break;
			case nsIWebDavResource.WEBDAV_ACTION_DELETE:
				error = "Unable to delete resource " + resource.path;
				break;
			case nsIWebDavResource.WEBDAV_ACTION_READ:
			default:
				return;
		}
		
		switch (errorCode) {
			case nsIWebDavResource.WEBDAV_ERROR_NO_ERROR:
				return;
				
			case nsIWebDavResource.WEBDAV_ERROR_UNKNOW:
				reason = "\nReason: unknow";
				break;
			case nsIWebDavResource.WEBDAV_ERROR_SERVER_UNAVAILABLE:
				reason = "\nReason: server unavailable";
				break;
			case nsIWebDavResource.WEBDAV_ERROR_INVALID_RESPONSE:
				reason = "\nReason: invalid response from server";
				break;
			case nsIWebDavResource.WEBDAV_ERROR_BAD_REQUEST:
				reason = "\nReason: bad request to server";
				break;
			case nsIWebDavResource.WEBDAV_ERROR_INSUFFICENT_STORAGE:
				reason = "\nReason: insufficent storage on server";
				break;
			case nsIWebDavResource.WEBDAV_ERROR_RESOURCE_ALREADY_EXISTS:
				reason = "\nReason: resource already exists";
				break;
			case nsIWebDavResource.WEBDAV_ERROR_PARENT_COLLECTION_NOT_FOUND:
				reason = "\nReason: one of parent collections is missing";
				break;
			case nsIWebDavResource.WEBDAV_ERROR_FORBIDDEN:
				reason = "\nReason: access forbidden";
				break;
				
			default:
				reason = "\nReason: unknow";
				break;
		}
		
		alert(error + reason);
	},

	// WebDavTreeItem
	
	get level() { return this._level; },
	get isCollection() { return this._webDavResource.isCollection; },
	get isCollectionEmpty() { return this._webDavResource.subResourcesCount == 0; },
	get webDavResource() { return this._webDavResource; },
	get opened() { return this._opened; },
	get parent() { return this._parent; },
	get removing() { return this._removing; },
	set removing(removing) {
		if (removing && this._removing)
			return;
		
		this._removing = removing;
		// if this resoruce is removing, any of the sub-resources doesn't need to be removed separately
		for (var i in this._items)
			this._items[i].removing = false;
	},
	
	get items() { return this._items; },
	
	/**
	 * Returns path relative to parent (or full path, if parent is null)
	 */
	get displayName() {
		if (this.parent)
			return this._webDavResource.path.substr(this.parent.webDavResource.path.length);
		else
			return this._webDavResource.path;
	},

	/**
	 * Returns size in kB, MB, GB or TB
	 */
	get size() {
		var size = this._webDavResource.size;
		if (size == -1)
			return "";
		
		suffix = "B";
		if (size > 1024) {
			size /= 1024;
			suffix = "kB";
		}
		if (size > 1024) {
			size /= 1024;
			suffix = "MB";
		}
		if (size > 1024) {
			size /= 1024;
			suffix = "GB";
		}
		if (size > 1024) {
			size /= 1024;
			suffix = "TB";
		}
		
		return size.toFixed(2) + " " + suffix;
	},

	/**
	 * Returns creation date of resource in iso 8601 format.
	 */
	get creationDate() { return this._webDavResource.creationDate; },
	/**
	 * Returns modification date of resource in iso 8601 format.
	 */
	get modificationDate() { return this._webDavResource.modificationDate; },
	/**
	 * Returns mime type of resource.
	 */
	get mimeType() { return this._webDavResource.mimeType; },
	
	/**
	 * Returns status text of resource. Splited into 2 parts: loaded and deleting.
	 */
	get status() {
		var loaded = "";
		switch (this._webDavResource.loaded) {
			case nsIWebDavResource.WEBDAV_LOADED_NOT_LOADED:
				loaded = "Not loaded";
				break;
			case nsIWebDavResource.WEBDAV_LOADED_LOADING:
				loaded = "Loading";
				break;
			case nsIWebDavResource.WEBDAV_LOADED_LOADED:
				loaded = "";
				break;
			case nsIWebDavResource.WEBDAV_LOADED_NOT_AVAILABLE:
				loaded = "Not available";
				break;
			case nsIWebDavResource.WEBDAV_LOADED_CREATING:
				loaded = "Creating";
				break;
			default:
				loaded = "";
		}
		
		var deleting = "";
		switch (this._webDavResource.deleting) {
			case nsIWebDavResource.WEBDAV_DELETING_NOT_DELETING:
				deleting = "";
				break;
			case nsIWebDavResource.WEBDAV_DELETING_DELETING:
				deleting = "Deleting";
				break;
			case nsIWebDavResource.WEBDAV_DELETING_DELETED:
				deleting = "Deleted";
				break;
			default:
				deleting = "";
		}
		
		if (loaded != "" && deleting != "")
			return loaded + ", " + deleting;
		else
			if (loaded != "")
				return loaded;
			else
				return deleting;
	},

	set index(index) { this._index = index; },
	get index() { return this._index; },

	get lastItem() {
		if (this._items.length > 0)
			return this._items[this._items.length - 1];
		else
			return null;
	},

	/**
	 * Returns count of sub items with count of sub items of sub items and so on.
	 */
	get subItemsCount() {
		var result = 0;
		for (var i in this._items) {
			result++;
			result += this._items[i].subItemsCount;
		}
		return result;
	},

	/**
	 * If it is added to tree view, and sub items too.
	 */
	addedToTreeView: function(treeView, level, index) {
		this._level = level;

		for (var i in this._items) {
			index++;
			treeView.addItem(level + 1, index, this._items[i]);
		}
	},

	toggleOpenState: function() {
		if (this._opened)
			this.close();
		else
			this.open();
	},
	
	subItemRemoved: function(item) {
		var index = this._items.indexOf(item);
		if (index != -1)
			this._items.splice(index, 1);
	},

	/**
	 * Opens a resource.
	 * If it is a non-collection, then redirect to this resoruce so it can be downloaded.
	 * If it is a collection then just get sub-resources and add TreeViewItems to tree.
	 */
	open: function() {
		if (this._opened)
			return;
		
		if (!this._webDavResource.isCollection) {
			document.location = "webdav://" + this._webDavResource.host + "/" + this._webDavResource.path;
			return;
		} 
		
		var subResources = this._webDavResource.getSubResources();
		var enumerator = subResources.enumerate();

		this._items = new Array();
		while (enumerator.hasMoreElements()) {
			var element = enumerator.getNext();
			element.QueryInterface(nsIWebDavResource);
			if (element)
				this._items[this._items.length] = new WebDavTreeItem(this._webDavTreeView, this, element);
		}
		this._sortItems();

		this._webDavTreeView.addItems(this._level + 1, this._index + 1, this._items);			
		this._opened = 1;
	},
	
	close: function() {
		// close removes sub-TreeViewItems, but not subResources
		this._webDavTreeView.removeItems(this._index + 1, this.subItemsCount);

		this._opened = 0;
		this._items = new Array();
	},
	
	setSorting: function(sortBy, sortIncreasing) {
		if ((this._sortBy == sortBy) && (this._sortIncreasing == sortIncreasing))
			return;
			
		this._sortBy = sortBy;
		this._sortIncreasing = sortIncreasing;
		
		this._sortItems();
	},
	
	_sortItems: function() {
		var item = this;
		
		this._items.sort(function(a, b) {
			return item._compareItems(item, a, b);
		});
	},
	
	_compareItems: function(item, a, b) {
		// collections go first
		if (!a.isCollection && b.isCollection)
			return 1;
		if (a.isCollection && !b.isCollection)
			return -1;
		
		var mul = 1;
		if (!item._sortIncreasing)
			mul = -1;

		switch (item._sortBy) {
			case "name":
				return mul * ((a.displayName > b.displayName) ? 1 : -1);
					
			case "type":
				if (a.mimeType == b.mimeType)
					return (a.displayName > b.displayName) ? 1 : -1;
				return mul * ((a.mimeType > b.mimeType) ? 1 : -1);
				
			case "size":
				if (a.webDavResource.size == b.webDavResource.size)
					return (a.displayName > b.displayName) ? 1 : -1;
				return mul * ((a.webDavResource.size > b.webDavResource.size) ? 1 : -1);
				
			case "creation-date":
				var acreationDate = Date.parse(a.creationDate);
				var bcreationDate = Date.parse(b.creationDate);
				if (acreationDate == acreationDate)
					return (a.displayName > b.displayName) ? 1 : -1;
				return mul * ((acreationDate > bcreationDate) ? 1 : -1);
				
			case "modification-date":
				var amodificationDate = Date.parse(a.modificationDate);
				var bmodificationDate = Date.parse(b.modificationDate);
				if (amodificationDate == bmodificationDate)
					return (a.displayName > b.displayName) ? 1 : -1;
				return mul * ((amodificationDate > bmodificationDate) ? 1 : -1);
		}
	}

};

function WebDavTreeItem(webDavTreeView, parent, webDavResource) {
	this._webDavTreeView = webDavTreeView;
	this._parent = parent;
	this._webDavResource = webDavResource;
	this._items = new Array();
	webDavResource.observer = this;
}
